﻿/*
 Author: Wojciech Domski
 Email:  Wojciech.Domski@gmail.com
 WWW:    Domski.pl

 copyright 2013 Wojciech Domski

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "sos.h"

/**
 * \brief Time
 *
 * Time in ticks
 */
volatile unsigned int sos_time;

/**
 * \brief Callback event array
 *
 */
volatile sos_function_callback_t sos_callback_array[SOS_MAX_EVENT_NUMBER];

/**
 * \brief Period array
 *
 * Period in ticks
 *
 * Accepted values:
 *  - 1 - event inactive
 * >= 1 - period in ticks
 */
volatile int sos_period[SOS_MAX_EVENT_NUMBER];

/**
 * \brief Counter
 *
 * When compared to sos_period value a
 * corresponding event's flag is set.
 */
volatile int sos_compare_counter[SOS_MAX_EVENT_NUMBER];

/**
 * \brief Event ready flag
 *
 * 1 - event ready
 * 0 - event waiting
 */
volatile int sos_flag[SOS_MAX_EVENT_NUMBER];

/**
 * \brief First free slot
 */
volatile int sos_slot;

void sos_init(void) {
	int i;

	//hardware dependent counter initialization
	//any counter which generates interrupt
	/*
	 //for STM32 with 72 MHz clock
	 if (SysTick_Config((CZESTOTLIWOSC_HZ / 8) / 1000)) {
	 while (1)
	 ;
	 }

	 SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK_Div8);
	 */

	sos_slot = 0;

	for (i = 0; i < SOS_MAX_EVENT_NUMBER; ++i) {
		sos_callback_array[i] = 0;
		sos_period[i] = SOS_EVENT_OFF;
		sos_compare_counter[i] = 0;
	}
}

void sos_handler(void) {
	int i;

	++sos_time;

	for (i = 0; i < SOS_MAX_EVENT_NUMBER; ++i) {
		if (sos_period[i] != SOS_EVENT_OFF) {
			++sos_compare_counter[i];

			if (sos_compare_counter[i] >= sos_period[i]) {
				sos_compare_counter[i] = 0;
				sos_flag[i] = 1;
			}
		}
	}
}

unsigned int sos_get_time(void) {
	return sos_time;
}

void sos_run(void) {
	int i;
	for (i = 0; i < SOS_MAX_EVENT_NUMBER; ++i) {
		if (sos_flag[i]) {
			sos_flag[i] = 0;
			sos_callback_array[i]();
		}
	}
}

int sos_add_event(sos_function_callback_t ptr, int period, int slot) {
	if (slot < 0) {
		slot = sos_slot;
		++sos_slot;
	}

	if (slot >= SOS_MAX_EVENT_NUMBER) {
		return -1;
	}

	sos_callback_array[slot] = ptr;
	sos_compare_counter[slot] = 0;
	sos_period[slot] = period;

	return slot;
}

void sos_set_period(int period, int slot) {
	sos_period[slot] = period;
	sos_compare_counter[slot] = 0;
}

